#define GLIBCXX_DEBUG
#include <array>
#include <cassert>
#include <iostream>
#include <vector>
using namespace std;

template <class T>
struct FenwickTree { // queries are right-exclusive; 0-based
	int n;
	vector<T> tree;
	FenwickTree(int n = 0) : n(n) { tree.assign(n + 1, 0); }
	T query(int r) { // [0,r]
		++r;
		T s = 0;
		for(; r > 0; r -= (r & (-r))) s += tree[r];
		return s;
	}
	void update(int i, T v) {
		for(++i; i <= n; i += (i & (-i))) tree[i] += v;
	}
};

const int MMAX = 1e6 + 1;

struct S {
	int pos, steps;
};

struct T {
	array<FenwickTree<int>, 2> ft;
	array<vector<S>, 2> diffs;
	T(int n) {
		for(auto &f : ft) f = FenwickTree<int>(MMAX / 2 + 1);

		// how many jumps do we need to go i to the left/right?
		diffs[0].push_back({0, 0});
		diffs[1].push_back({-1, 0});
		for(int i = 1, dist = 1; dist <= MMAX + 4000; ++i, dist += i)
			diffs[dist % 2].push_back({dist, i});
	}

	void update(int pos, int delta) {
		for(auto p : {0, 1}) {
			auto q = (p + pos) % 2;
			// to the right
			int to_update = 0;
			for(size_t i = 0; i + 1 < diffs[p].size() && pos + diffs[p][i].pos < MMAX; ++i) {
				// the +2 must be within the division, to make sure -1 is rounded the right way
				int l = (pos + diffs[p][i].pos + 2) / 2;
				int r = (pos + diffs[p][i + 1].pos) / 2 + 1;
				int s = diffs[p][i + 1].steps;
				ft[q].update(l, delta * s + to_update);
				if(r <= MMAX / 2)
					to_update = -delta * s;
				else
					to_update = 0;
			}
			assert(to_update == 0);

			// to the left
			for(size_t i = 0; i + 1 < diffs[p].size() && pos - diffs[p][i].pos > 1; ++i) {
				int l = (pos - diffs[p][i + 1].pos) / 2;
				int r = (pos - diffs[p][i].pos) / 2;
				int s = diffs[p][i + 1].steps;
				ft[q].update(r, -delta * s + to_update);
				ft[q].update(max(l, 0), delta * s + to_update);
			}
		}
	}

	int query(int pos) { return ft[pos % 2].query(pos / 2); }
};

int main() {
	int n, p;
	cin >> n >> p;
	T t(n);
	while(n--) {
		int pos;
		cin >> pos;
		t.update(pos, 1);
	}
	int q;
	cin >> q;
	while(q--) {
		char c;
		cin >> c;
		int pos;
		cin >> pos;
		if(c == 't')
			p = pos;
		else if(c == '+')
			t.update(pos, 1);
		else if(c == '-')
			t.update(pos, -1);
		else
			assert(false);
		cout << t.query(p) << '\n';
	}
	return 0;
}
